from models.span_context import SpanContext
from torch_client import TorchClient

from acceldata_airflow_sdk.decorators.span import span
from acceldata_airflow_sdk.operators.span_operator import SpanOperator

torch_client = TorchClient(url="https://torch.acceldata.local:5443",
                           access_key="N1LTYRK630PZ", secret_key="xPeUj4Iyj4WL2Tw284s9mqsgxvbPKW")
pipeline_uid = 'talend.etl.demo'

pipeline = torch_client.get_pipeline(pipeline_uid)


pipelineRunResponse = pipeline.create_pipeline_run(context_data={'team': 'acceldata_sdk_code', 'name': 'backend'})
spanContext = pipelineRunResponse.create_span(uid='parent_span',
                                              context_data={'start': 'tuesday', 'name': 'vaishvik'})

pipeline_run = pipeline.get_latest_pipeline_run()
span_context = pipeline_run.get_span(span_uid='parent_span')
print('done1')

parent_span_context = span_context
first_span_context = parent_span_context.create_child_span(uid='first_span',
                                                         context_data={'time': 'abcd'})

parent_span_context_dict = parent_span_context.__dict__
print('done2')
get_parent_span_context = SpanContext(**parent_span_context_dict)
second_span_context = get_parent_span_context.create_child_span(uid='second_span',
                                                         context_data={'time': 'def'})



import requests
import json
from datetime import datetime
from dataclasses import dataclass, asdict
from acceldata_sdk.torch_client import TorchClient


@dataclass
class Pipelineb:
    id: str

    def __init__(self, dag_id, **kwargs):
        self.id = dag_id


@dataclass
class PipelineRunb:
    id: str
    pipelineId: str
    state: str
    start_date: datetime
    end_date: datetime

    def __init__(self, dag_run_id, dag_id, state, start_date, end_date, **kwargs):
        self.id = dag_run_id
        self.pipelineId = dag_id
        self.state = "SUCCESS"
        if state is not None and 'failed' in state:
            self.state = 'FAILURE'
        # elif state is not None:
        #     self.state = state
        self.start_date = start_date
        self.end_date = end_date


@dataclass
class Jobb:
    id: str
    downstreamTaskIds: [str]
    state: str
    start_date: datetime
    end_date: datetime
    pipelineId: str
    pipelineRunId: str

    def __init__(self, task_id, dag_id, start_date,end_date, dag_run_id,state="STARTED", downstream_task_ids = [], **kwargs):
        self.id = task_id
        self.pipelineId = dag_id
        self.state = "FINISHED"
        if state is not None and 'fail' in state:
            self.state = 'ERRORED'
        # elif state is not None:
        #     self.state = state
        self.start_date = start_date
        self.end_date = end_date
        self.pipelineRunId = dag_run_id
        self.downstreamTaskIds = downstream_task_ids


@dataclass
class PipelineBulkupdateRequest:
    pipelines: list()
    pipelineruns: list()
    jobs: list()


def collect_data():
    # URL = "http://airflow_airflow-webserver_1:8080/api/v1/dags"
    DAGS_URL = "http://localhost:8082/api/v1/dags"

    PARAMS = {}
    _HEADERS = dict()
    _HEADERS['Authorization'] = 'Basic YWlyZmxvdzphaXJmbG93'
    # sending get request and saving the response as response object
    r = requests.get(url = DAGS_URL, params = PARAMS, headers = _HEADERS)


    # extracting data in json format
    data = r.json()
    print (data)
    dags = data['dags']
    dagruns = list()
    dagidTaskid_downstream = dict()
    for dag in dags:
        tasksperdags = dict()
        DAGSRUNS_URL = f"http://localhost:8082/api/v1/dags/{dag['dag_id']}/dagRuns"
        r = requests.get(url=DAGSRUNS_URL, params=PARAMS, headers=_HEADERS)
        runs = r.json()
        dagruns.extend(runs['dag_runs'])
        DAGStasks_URL = f"http://localhost:8082/api/v1/dags/{dag['dag_id']}/tasks"
        r = requests.get(url=DAGStasks_URL, params=PARAMS, headers=_HEADERS)
        if r.status_code == 200:
            data = r.json()
            tasksperdags[dag['dag_id']] = data['tasks']
            for task in tasksperdags[dag['dag_id']]:
                dagid_taskid = f"{dag['dag_id']}:{task['task_id']}"
                dagidTaskid_downstream[dagid_taskid] = task['downstream_task_ids']

    tasks = list()
    for dagrun in dagruns:
        DAGSRUNStasks_URL = f"http://localhost:8082/api/v1/dags/{dagrun['dag_id']}/dagRuns/{dagrun['dag_run_id']}/taskInstances"
        r = requests.get(url=DAGSRUNStasks_URL, params=PARAMS, headers=_HEADERS)
        tasks_instances = r.json()
        for task_instance in tasks_instances['task_instances']:
            task_instance['dag_run_id'] = dagrun['dag_run_id']
            try:
                task_instance['downstream_task_ids'] = dagidTaskid_downstream[f"{dagrun['dag_id']}:{task_instance['task_id']}"]
            except:
                print ("key not found")
        tasks.extend(tasks_instances['task_instances'])

    temp_pipelines = list()
    temp_pipelineruns = list()
    temp_jobs = list()
    for temp in dags:
        temp_pipelines.append(Pipelineb(**temp))
    for temp in dagruns:
        temp_pipelineruns.append(PipelineRunb(**temp))
    for temp in tasks:
        temp_jobs.append(Jobb(**temp))
    print (tasks_instances)
    bulkUpdateRequest = PipelineBulkupdateRequest(pipelines = temp_pipelines, pipelineruns = temp_pipelineruns, jobs = temp_jobs)
    print (bulkUpdateRequest)
    print (f'json: {bulkUpdateRequest}')

    print(json.dumps(asdict(bulkUpdateRequest)))
    torch_credentials = {
        'url': 'https://torch.acceldata.local:5443/torch',
        'access_key': 'P04IM8FNQRUCRTU',
        'secret_key': 'E6LL9YUPMG4BDTJHT2VZD75HW0B8E5'
    }

    torch_client = TorchClient(**torch_credentials)
    torch_client.bulkupdate_pipeline(bulkUpdateRequest)


if __name__ == '__main__':
    collect_data()



import logging
import time
from abc import ABC
from typing import List

import requests

from acceldata_sdk.models.connection import ConnectionType, AnalyticsPipeline, Connection, ConnectionCheck, ConnectionCheckStatus
from acceldata_sdk.models.profile import Profile, ProfileRequest
from acceldata_sdk.models.rule import Rule, RuleCancelResponse, RuleExecution, ExecutionResult
from acceldata_sdk.models.tags import TAG, AssetTag, AssetLabel
from acceldata_sdk.errors import APIError, TorchSdkException
from acceldata_sdk.models.assetType import AssetType
from acceldata_sdk.models.datasource import DataSource,SourceType, CrawlerStatus
from acceldata_sdk.models.job import Job
from acceldata_sdk.models.pipeline import CreatePipeline, Pipeline, PipelineRun, PipelineInfo, PipelineDetails
from acceldata_sdk.models.span import Span, SpanContextEvent, CreateSpanEvent
from acceldata_sdk.client import TorchClientInterFace
from acceldata_sdk.models.asset import Asset, Metadata
from requests_toolbelt import MultipartEncoder
from acceldata_sdk.models.dqrule import DataQualityExecutionResult, DataQualityRule, DataQualityRuleResource
from acceldata_sdk.models.reconcillationrule import ReconcillationExecutionResult, ReconciliationRuleResource
from acceldata_sdk.models.ruleExecutionResult import RuleResult, PolicyFilter, ExecutionPeriod, RuleType, RuleExecutionSummary
from acceldata_sdk.constants import AssetSourceType, SdkSupportedVersions, TorchBuildVersion
from acceldata_sdk.common import Executor
from acceldata_sdk.constants import PolicyType, FailureStrategy, RuleExecutionStatus, PipelineBulkupdateRequest
from semantic_version import Version, SimpleSpec


from typing import List
from dataclasses import asdict, dataclass
import json
from datetime import datetime, timedelta

_HEADERS = {'User-Agent': 'Acceldata-sdk', 'accessKey': None, 'secretKey': None, 'Content-Type': 'application/json'}
catalog_api_path = "/catalog-server/api"
pipeline_api_path = "/torch-pipeline/api"
admin_api_path = "/admin/api"

import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def check_int(input_str):
    try:
        int(input_str)
        return True
    except ValueError:
        return False


class TorchHttpClient(TorchClientInterFace, ABC):
    access_key = None
    secret_key = None
    logger = logging.getLogger('torch')
    logger.setLevel(logging.INFO)

    # Create torch client by passing secret and access keys of catalog server for a given url
    def __init__(self, url, timeout_ms=10000, access_key: str = None, secret_key: str = None):
        """
            Description : Torch client is used to send data to catalog server.

            :param url: (String) url of the catalog server
                        ex:  https://torch.acceldata.local:5443
            :param timeout_ms: (Integer) timeout of the requests sending to catalog
            :param access_key: (String) Access key of API key. You can generate API key from torch ui's setting
            :param secret_key: (String) Secret key of API key.

            : TorchClient = TorchClient(url='https://torch.acceldata.local:5443', access_key='OY2VVIN2N6LJ', secret_key='da6bDBimQfXSMsyyhlPVJJfk7Zc2gs')
        """
        self.timeout_ms = timeout_ms
        self._timeout = 10000
        self._catalog_api_base = f"{url}{catalog_api_path}"
        self._pipeline_api_base = f"{url}{pipeline_api_path}"
        loc = url.rfind('/torch')
        if loc > 0:
            self._admin_api_base = f"{url[:loc]}{admin_api_path}"
        else:
            self._admin_api_base = f"{url}{admin_api_path}"

        if access_key is None and secret_key is None:
            raise Exception('Access key and secret key - required')
        self.access_key = access_key
        self.secret_key = secret_key
        _HEADERS['accessKey'] = access_key
        _HEADERS['secretKey'] = secret_key

    # # convert pipeline object to dict type
    def _convert_pipeline_to_dict(self, pipeline: CreatePipeline):
        """
        Description:
            Convert pipeline object to dict type.
        :param pipeline: createPipeline class instance
        :return: pipeline in dict type
        """
        pipeline_dict = pipeline.__dict__
        if pipeline.meta is not None:
            meta = asdict(pipeline.meta)
            pipeline_dict['meta'] = meta
        return pipeline_dict

    def get_supported_sdk_versions(self) -> SdkSupportedVersions:
        response = self._get(
            f'{self._catalog_api_base}/sdk/compatible-versions'
        )
        return SdkSupportedVersions(**response['versions'])

    def get_torch_version(self) -> TorchBuildVersion:
        response = self._get(
            f'{self._admin_api_base}/build-details'
        )
        return TorchBuildVersion(**response)

    # function to create pipeline
    def create_pipeline(self, pipeline: CreatePipeline) -> Pipeline:
        """
        Description:
            Used to create pipeline
        :param pipeline: createPipeline class instance that you want to create
        :return: pipeline class instance
        """
        payload = self._convert_pipeline_to_dict(pipeline)
        pipeline_payload = {'pipeline': payload}
        response = self._put(
            f'{self._pipeline_api_base}/pipelines',
            payload=pipeline_payload
        )
        response['pipeline']['client'] = self
        self.logger.info('Pipeline Created')
        return Pipeline(**response['pipeline'])


    def bulkupdate_pipeline(self, bulkUpdateRequest: PipelineBulkupdateRequest):
        """
        Description:
            Used to bulk update pipeline
        :param pipeline: createPipeline class instance that you want to create
        :return: pipeline class instance
        """
        payload = asdict(bulkUpdateRequest)
        response = self._post(
            f'{self._pipeline_api_base}/pipelines/bulkupdate',
            payload=payload, as_json=False
        )
        # response['pipeline']['client'] = self
        self.logger.info(f'Pipelines Created')
        return response

    # function to delete pipeline
    def delete_pipeline(self, id):
        """
        Description:
            Used to delete pipeline
        :param id: id of pipeline
        """

        response = self._delete(
            f'{self._pipeline_api_base}/pipelines/{id}'
        )
        self.logger.info(f'Pipeline {id} deleted')
        return response

    # get pipeline by uid or id
    def get_pipeline(self, pipeline_identity) -> Pipeline:
        """
            Description:
                To get an existing pipeline from torch catalog
        :param pipeline_identity: uid or id of the pipeline
        :return:(Pipeline) pipeline class instance
        """
        url = f'{self._pipeline_api_base}/pipelines/{pipeline_identity}'
        response = self._get(url)
        response['pipeline']['client'] = self
        return Pipeline(**response['pipeline'])

    # get all pipelines
    def get_pipelines(self) -> List[PipelineInfo]:
        """
            Description:
                To get all pipelines from torch catalog
        :return:(List[PipelineInfo]) PipelineInfo class instance
        """
        url = f'{self._pipeline_api_base}/pipelines'
        response = self._get(url)
        pipelines_infos = list()
        pipelines = response['pipelines']
        for obj in pipelines:
            obj['pipeline']['client'] = self
            pipelines_infos.append(PipelineInfo(pipeline=obj['pipeline'], runs=obj['runs']))

        return pipelines_infos

    # to create job for any given pipeline
    def create_job(self, job: {}, pipelineId: int) -> Job:
        """
        Description:
            Used to create job in a pipeline
        :param pipelineId: pipeline id of the respective pipeline
        :param job: createJob class instance that you want to add in pipeline
        :return: Job class instance of created job
        """
        response = self._put(
            f'{self._pipeline_api_base}/pipelines/{pipelineId}/jobs',
            payload=job
        )
        self.logger.info('Pipeline Job Created')
        return Job(**response['node'])

    def delete_datasource(self, datasource_id: int):
        """
        Description:
            used to delete datasource
        :param datasource_id: id of the datasource to be updated
        :return: response of the operation
        """
        response = self._delete(
            f'{self._catalog_api_base}/assemblies/{datasource_id}'
        )
        return response

    def start_crawler(self, datasource_name) -> CrawlerStatus:
        response = self._post(
            f'{self._catalog_api_base}/crawler/{datasource_name}',
            payload={}
        )
        return CrawlerStatus(**response['data'])

    def get_crawler_status(self, datasource_name) -> CrawlerStatus:
        response = self._get(
            f'{self._catalog_api_base}/crawler/{datasource_name}',
        )
        return CrawlerStatus(**response['data'])

    # create run for a pipeline
    def create_pipeline_run(self, pipeline_run: {}) -> PipelineRun:
        """
        Description:
            used to create a pipeline run
        :param pipeline_run:
        :return: pipelineRun class instance
        """
        pipelineId = pipeline_run['run']['pipelineId']
        response = self._post(
            f'{self._pipeline_api_base}/pipelines/{pipelineId}/runs',
            payload=pipeline_run
        )
        response['run']['client'] = self
        self.logger.info('Pipeline Run Created')
        return PipelineRun(**response['run'])

    # update run for a pipeline
    def update_pipeline_run(self, pipeline_run_id: int, pipeline_run: {}) -> PipelineRun:
        """
        Description:
            used to update an existing pipeline run
        :param pipeline_run_id: pipeline run id that you want to update
        :param pipeline_run: pipelineRun class instance that you want to update
        :return: updated pipelineRun class instance
        """
        response = self._put(
            f'{self._pipeline_api_base}/pipelines/runs/{pipeline_run_id}',
            payload=pipeline_run
        )
        response['run']['client'] = self
        self.logger.info('Pipeline Run Updated')
        return PipelineRun(**response['run'])

    # get latest pipeline run
    def get_latest_pipeline_run(self, pipeline_id: int) -> PipelineRun:
        """
            Description:
                To get latest pipeline run instance of any pipeline
        :param pipeline_id: id of the pipeline
        :return: PipelineRun instance
        """
        url = f'{self._pipeline_api_base}/pipelines/{pipeline_id}/latestRun'
        response = self._get(url)
        response['run']['client'] = self
        return PipelineRun(**response['run'])

    # get pipeline run with particular continuation id or run id
    def get_pipeline_run(self, pipeline_run_id: int = None, continuation_id: str = None, pipeline_id: str = None) -> PipelineRun:
        """
            Description:
                To get pipeline run with particular id or continuation id of any pipeline
        :param pipeline_run_id: run id of the pipeline run
        :param continuation_id: continuation id of the pipeline run
        :param pipeline_id: id of the pipeline. This is a mandatory parameter when run is being queried using continuation_id
        :return: PipelineRun instance
        """
        payload = dict()
        return_run = dict()
        if pipeline_run_id is not None:
            url = f'{self._pipeline_api_base}/pipelines/runs/{pipeline_run_id}'
            response = self._get(url)
            return_run = response['run']
        elif continuation_id is not None and pipeline_id is not None:
            supported_versions = self.get_supported_sdk_versions()
            torch_version: str = self.get_torch_version()
            if torch_version.buildVersion is not None and torch_version.buildVersion != "":
                actual_version = torch_version.buildVersion[:5]
            else:
                actual_version = supported_versions.maxVersion
            ver_comparator = SimpleSpec(f'<={actual_version}')
            if Version('2.5.0') in ver_comparator:
                url = f'{self._pipeline_api_base}/pipelines/{pipeline_id}/runs'
                payload = {"continuationId": continuation_id}
                response = self._get(url, params=payload)
                return_run = response["runs"][0]
            else:
                # For torch versions lower than 2.5.0 use old call for continuation_id
                if continuation_id.isdigit() is False:
                    url = f'{self._pipeline_api_base}/pipelines/runs/{continuation_id}'
                    response = self._get(url)
                    return_run = response['run']
                else:
                    raise TorchSdkException('Please provide continuation_id which is not a valid number.')
        else:
            raise TorchSdkException('Please provide either continuation_id and pipeline_id or pipeline_run_id')

        return_run['client'] = self
        return PipelineRun(**return_run)

    # get pipeline runs for a pipeline
    def get_pipeline_runs(self, pipeline_id: int) -> List[PipelineRun]:
        """
            Description:
                To get pipeline run  id of any pipeline
        :param pipeline_id: id of the pipeline
        :return: List[PipelineRun] instance
        """
        url = f'{self._pipeline_api_base}/pipelines/{pipeline_id}/runs'
        response = self._get(url)
        runs = list(response['runs'])
        runs_list = []
        for res in runs:
            res['client'] = self
            runs_list.append(PipelineRun(**res))
        return runs_list

    # create span for pipeline run
    def create_span(self, pipeline_run_id: int, span: dict) -> Span:
        """
        Description:
            used to create span for any pipeline run
        :param pipeline_run_id:
        :param span: Span class instance
        :return: Span
        """
        response = self._post(
            f'{self._pipeline_api_base}/pipelines/runs/{pipeline_run_id}/spans',
            payload=span
        )
        self.logger.info('Span Created')
        return Span(**response['span'])

    def get_child_spans(self, span_id: int):
        response = self._get(
            f'{self._pipeline_api_base}/pipelines/spans/{span_id}/childSpans'
        )
        self.logger.info('Fetched child spans.')
        child_spans = list(response['childSpans'])
        child_spans_res = []
        for res in child_spans:
            child_spans_res.append(Span(**res))
        return child_spans_res

    def get_span(self, pipeline_run_id: int, uid: str) -> Span:
        """
            Description:
                Get span of the pipeline run by uid
        :param pipeline_run_id: pipeline run id
        :param uid: uid of the span
        :return: SpanContext instance of the input span uid
        """
        url = f'{self._pipeline_api_base}/pipelines/runs/{pipeline_run_id}/spans/{uid}'
        response = self._get(url)
        response['span']['client'] = self
        return Span(**response['span'])

    def get_root_span(self, pipeline_run_id: int) -> Span:
        """
            Description:
                Get root span of the pipeline run
        :param pipeline_run_id: pipeline run id
        :return: SpanContext instance of the root span
        """
        url = f'{self._pipeline_api_base}/pipelines/runs/{pipeline_run_id}/spans'
        params = {"onlyRootSpan": "true"}
        response = self._get(url, params=params)
        print(f'response: {response}')
        return_value = dict()
        return_value['span'] = dict()
        return_value['span']['client'] = self
        if response['spans']:
            return_value['span'] = response['spans'][0]
            return_value['span']['client'] = self
            return Span(**return_value['span'])
        else:
            return None

    def get_spans(self, pipeline_run_id):
        response = self._get(
            f'{self._pipeline_api_base}/pipelines/runs/{pipeline_run_id}/spans'
        )
        self.logger.info('Fetched spans.')
        spans = list(response['spans'])
        spans_res = []
        for res in spans:
            spans_res.append(Span(**res))
        return spans_res

    # convert span object to dict type
    def convert_span_event_to_dict(self, span_event: CreateSpanEvent):
        """
            Description:
                Convert CreateSpanEvent class instance to dict type
            :param spanEvent: CreateSpanEvent class instance
            :return: dict form of CreateSpanEvent class instance
        """
        payload = span_event.__dict__
        # payload['eventUid'] = span_event.eventType
        event_payload = {'event': payload}
        return event_payload

    # create span event for any span
    def create_span_event(self, span_event: CreateSpanEvent) -> SpanContextEvent:
        """
        Description:
            used to create span event
        :param span_event: CreateSpanEvent class instance that you want to create
        :return: SpanContextEvent class instance
        """
        payload = self.convert_span_event_to_dict(span_event)
        if span_event.spanId is None:
            Exception('To update a pipeline run, id is required.')
        response = self._post(
            f'{self._pipeline_api_base}/pipelines/spans/{span_event.spanId}/events',
            payload=payload
        )
        self.logger.info('Span Event Created')
        return SpanContextEvent(**response['event'])

    def get_run_details(self, pipeline_id: int, version_id: int):
        """
            Description:
                Get details of the pipeline by versionid
        :param pipeline_id: pipeline  id
        :param version_id: version_id of the pipeline
        :return: SpanContext instance of the input span uid
        """
        url = f'{self._pipeline_api_base}/pipelines/{pipeline_id}/details?versionId={version_id}'
        response = self._get(url)
        details = response['details']
        details['pipeline']['client'] = self
        return PipelineDetails(**details)

    def get_all_asset_types(self):
        """
        Description:
            get all asset types supported in torch xatalog
        :return: list of asset types
        """
        response = self._get(
            f'{self._catalog_api_base}/asset-types'
        )
        asset_types = list(response['data'])
        asset_types_res = []
        for res in asset_types:
            asset_types_res.append(AssetType(**res))
        return asset_types_res

    def get_all_source_types(self):
        """
        Description:
            get all source types supported in torch catalog
        :return: list of all source type
        """
        response = self._get(
            f'{self._catalog_api_base}/source-types'
        )
        source_types = list(response['data'])
        source_types_res = []
        for res in source_types:
            source_types_res.append(SourceType(**res))
        return source_types_res

    def get_connection_types(self):
        """
        Description:
            get all source types supported in torch catalog
        :return: list of all source type
        """
        response = self._get(
            f'{self._catalog_api_base}/connection-types'
        )
        connection_types = list(response['data'])
        conn_type_response = []
        for res in connection_types:
            conn_type_response.append(ConnectionType(**res))
        return conn_type_response

    def get_tags(self):
        response = self._get(
            f'{self._catalog_api_base}/assets/tags'
        )
        tags = list(response['tags'])
        tags_list = []
        for res in tags:
            tags_list.append(TAG(**res))
        return tags_list

    def get_datasource(self, assembly_identifier, properties: bool = False) -> DataSource:
        """
        Description:
            Find datasource by its name or id in torch catalog
        :param assembly_identifier: name or id of the datasource given in torch
        :param properties: optional parameter, bool, to get datasource properties as well
        :return: (DataSource) datasource
        """
        url = f'{self._catalog_api_base}/assemblies/{assembly_identifier}?properties={properties}'

        response = self._get(url)
        datasource = response['data']
        if len(datasource) > 0:
            datasource['client'] = self
            return DataSource(**datasource)
        raise Exception('Datasource not found.')

    def get_datasources(self, type: AssetSourceType = None) -> List[DataSource]:
        """
        Description:
            Find datasources by its type in torch catalog
        :param type: type of the datasource given in torch, optional
        :return: list(DataSource) datasource
        """
        url = f'{self._catalog_api_base}/assemblies'
        if type is not None:
            params = dict()
            params['sourceType'] = type.name
            response = self._get(url, params=params)
        else:
            response = self._get(url)
        datasources = list(response['data'])
        ds_list = []
        for res in datasources:
            res['client'] = self
            ds_list.append(DataSource(**res))
        return ds_list
        raise Exception('Datasource not found.')

    def get_all_datasources(self):
        """
        Description:
            list all datasources in torch catalog
        :return: (DataSource) list of datasource
        """
        url = f'{self._catalog_api_base}/assemblies'
        response = self._get(url)
        datasources = list(response['data'])
        ds_list = []
        for res in datasources:
            ds_list.append(DataSource(**res))
        return ds_list

    def get_asset(self, identifier):
        """"
            Description:
                Find an asset of the datasource
            :param identifier: uid or ID of the asset
        """
        if check_int(identifier):
            return self.get_asset_by_id(id=identifier)
        else:
            return self.get_asset_by_uid(uid=identifier)

    def get_asset_by_id(self, id: int) -> Asset:
        """
        Description:
            used to find an asset
        :param id: id of an asset
        :return: asset data (Asset)
        """
        if id is None:
            raise Exception('Asset id is required')
        url = f'{self._catalog_api_base}/assets?ids={id}'
        asset = self._get_asset(url)
        asset_res = list(asset['data'])[0]
        asset_res['client'] = self
        return Asset(**asset_res)

    def get_asset_by_uid(self, uid: str):
        """
        Description:
            used to find an asset
        :param uid: uid of an asset
        :return: asset data (Asset)
        """
        if uid is None:
            raise Exception('Asset uid is required')
        url = f'{self._catalog_api_base}/assets?uid={uid}'
        asset = self._get_asset(url)
        asset['data']['client'] = self
        return Asset(**asset['data'])

    def _get_asset(self, url: str):
        response = self._get(
            url=url
        )
        return response

    def get_asset_metadata(self, id: int):
        url = f'{self._catalog_api_base}/assets/{id}/metadata'
        response = self._get(
            url=url
        )
        metadata = Metadata(**response['data'])
        return metadata

    def add_custom_metadata(self, id: int, custom_metadata: dict):
        url = f'{self._catalog_api_base}/assets/{id}/metadata/custom'
        response = self._post(
            url=url,
            payload=custom_metadata
        )
        metadata = Metadata(**response['data'])
        return metadata

    def profile_asset(self, id: int, payload: dict) -> Profile:
        url = f'{self._catalog_api_base}/assets/{id}/profile'
        response = self._post(
            url=url,
            payload=payload
        )
        response['data']['client'] = self
        profile_response = Profile(**response['data'])
        return profile_response

    def sample_data(self, id: int):
        url = f'{self._catalog_api_base}/assets/{id}/sample'
        response = self._get(
            url=url
        )
        return response

    def get_profile_request_details(self, asset_id: int, req_id: int):
        url = f'{self._catalog_api_base}/assets/{asset_id}/profile/{req_id}'
        response = self._get(
            url=url
        )
        return response

    def get_latest_profile_status(self, id: int):
        url = f'{self._catalog_api_base}/assets/{id}/profile'
        response = self._get(
            url=url
        )
        response['data']['client'] = self
        profile_response = ProfileRequest(**response['data'])
        return profile_response

    def cancel_profile(self, profile_req_id: int):
        url = f'{self._catalog_api_base}/assets/profile/{profile_req_id}/cancel'
        response = self._put(
            url=url,
            payload={}
        )
        return response['result']

    def get_asset_tags(self, id: int):
        if id is None:
            raise Exception('Asset id is required')
        url = f'{self._catalog_api_base}/assets/{id}/tags'
        response = self._get(
            url=url
        )
        tags = list(response['assetTags'])
        tags_list = []
        for res in tags:
            tags_list.append(AssetTag(**res))
        return tags_list

    def add_asset_tag(self, id: int, payload: dict):
        if id is None:
            raise Exception('Asset id is required')
        url = f'{self._catalog_api_base}/assets/{id}/tag'
        response = self._post(
            url=url, payload=payload
        )
        return AssetTag(**response)

    def add_asset_labels(self, id: int, payload: dict):
        if id is None:
            raise Exception('Asset id is required')
        url = f'{self._catalog_api_base}/assets/{id}/labels'
        response = self._put(
            url=url, payload=payload
        )
        labels = list(response)
        labels_list = []
        for lbl in labels:
            labels_list.append(AssetLabel(**lbl))
        return labels_list

    def get_asset_labels(self, id: int):
        if id is None:
            raise Exception('Asset id is required')
        url = f'{self._catalog_api_base}/assets/{id}/labels'
        response = self._get(
            url=url
        )
        labels = list(response)
        labels_list = []
        for lbl in labels:
            labels_list.append(AssetLabel(**lbl))
        return labels_list

    def get_analysis_pipeline(self, id: int) -> AnalyticsPipeline:
        response = self._get(
            url=f'{self._catalog_api_base}/analytics-pipelines/{id}'
        )
        return AnalyticsPipeline(**response['pipeline'])

    def check_connection(self, create_connection):
        create_conn = create_connection.__dict__
        config_prop = []
        for cp in create_connection.properties:
            config_prop.append(asdict(cp))
        create_conn['properties'] = config_prop
        payload = {
            'connection': create_conn
        }
        url = f'{self._catalog_api_base}/connections/check'
        response = self._post_form_data(
            url=url,
            payload=payload
        )
        check_res = ConnectionCheck(**response['result'])
        if check_res.status == ConnectionCheckStatus.SUCCESS.value:
            return check_res
        if check_res.status == ConnectionCheckStatus.FAILURE.value:
            raise TorchSdkException(f'Error: {check_res.message}')
        return check_res

    def create_connection(self, create_connection) -> Connection:
        create_conn = create_connection.__dict__
        config_prop = []
        # for cp in create_connection.properties:
        #     config_prop.append(asdict(cp))
        # create_conn['properties'] = config_prop
        payload = {
            'connection': create_conn
        }
        url = f'{self._catalog_api_base}/connections'
        response = self._post_form_data(
            url=url,
            payload=payload
        )
        return Connection(**response['connection'])

    def enable_rule(self, rule_id) -> Rule:
        payload = {
            "enable": True
        }
        response = self._put(
            url=f'{self._catalog_api_base}/rules/{rule_id}/enable',
            payload=payload
        )
        return Rule(**response['rule'])

    def disable_rule(self, rule_id) -> Rule:
        payload = {
            "enable": False
        }
        response = self._put(
            url=f'{self._catalog_api_base}/rules/{rule_id}/enable',
            payload=payload
        )
        return Rule(**response['rule'])

    def cancel_rule_execution(self, execution_id) -> RuleCancelResponse:
        response = self._put(
            url=f'{self._catalog_api_base}/rules/execution/{execution_id}/cancel',
            payload={}
        )
        return RuleCancelResponse(**response['result'])

    def execute_rule(self, rule_type: PolicyType, rule_id, sync=True, incremental=False,
                     failure_strategy: FailureStrategy = FailureStrategy.DoNotFail, pipeline_run_id=None):
        """
        Description:
            To execute rule synchronously and asynchronously
        :param rule_type: (PolicyType) Type of rule to be executed
        :param rule_id: (String) id of the rule to be executed
        :param sync: (bool) optional Set it to False if asynchronous execution has to be done
        :param incremental: (bool) optional Set it to True if full execution has to be done
        :param failure_strategy: (enum) optional Set it to decide if it should fail at error,
            fail at warning or never fail
        """
        rule_executor = Executor(rule_type, self, sync=sync)
        rule_executor.execute(rule_id, incremental, failure_strategy=failure_strategy, pipeline_run_id=pipeline_run_id)
        return rule_executor

    def get_rule_status(self, rule_type, execution_id) -> RuleExecutionStatus:
        """
        Description:
            To get status of rule execution
        :param rule_type: (String) Type of rule to be executed
        :param execution_id: (String) ID of execution of the rule previously executed.
        """
        rule_executor = Executor(rule_type, self)
        return rule_executor.get_execution_status(execution_id)

    def get_rule_execution_result(self, rule_type, execution_id, failure_strategy: FailureStrategy = FailureStrategy.DoNotFail):
        """
        Description:
            To get result of rule execution
        :param rule_type: (String) Type of rule to be executed
        :param execution_id: (String) ID of execution of the rule previously executed.
        :param failure_strategy: (enum) optional Set it to decide if it should fail at error,
            fail at warning or never fail
        """
        rule_executor = Executor(rule_type, self)
        return rule_executor.get_execution_result(execution_id, failure_strategy=failure_strategy)

    def execute_dq_rule(self, rule_id, incremental=False, pipeline_run_id=None) -> RuleExecution:
        payload_temp = dict()
        payload_temp['executionType'] = "FULL"
        if incremental:
            payload_temp['executionType'] = "INCREMENTAL"
        if pipeline_run_id is not None:
            payload_temp['pipelineRunId'] = pipeline_run_id
        response = self._post(
            url=f'{self._catalog_api_base}/rules/data-quality/{rule_id}/executions',
            payload=payload_temp
            )
        return RuleExecution(**response['execution'])

    def execute_reconciliation_rule(self, rule_id, incremental=False, pipeline_run_id=None) -> RuleExecution:
        payload_temp = dict()
        payload_temp['executionType'] = "FULL"
        if incremental:
            payload_temp['executionType'] = "INCREMENTAL"
        if pipeline_run_id is not None:
            payload_temp['pipelineRunId'] = pipeline_run_id
        response = self._post(
            url=f'{self._catalog_api_base}/rules/reconciliation/{rule_id}/executions',
            payload=payload_temp
        )
        return RuleExecution(**response['execution'])

    def get_dq_rule(self, rule_id) -> DataQualityRuleResource:
        response = self._get(
            url=f'{self._catalog_api_base}/rules/data-quality/{rule_id}'
            )
        response['client'] = self
        return DataQualityRuleResource(**response)

    def get_reconciliation_rule(self, rule_id) -> ReconciliationRuleResource:
        response = self._get(
            url=f'{self._catalog_api_base}/rules/reconciliation/{rule_id}'
        )
        response['client'] = self
        return ReconciliationRuleResource(**response)

    def get_dq_rule_execution_details(self, execution_id) -> ExecutionResult:
        response = self._get(
            url=f'{self._catalog_api_base}/rules/data-quality/executions/{execution_id}'
        )
        return ExecutionResult(**response)

    def get_reconciliation_rule_execution_details(self, execution_id) -> ExecutionResult:
        response = self._get(
            url=f'{self._catalog_api_base}/rules/reconciliation/executions/{execution_id}'
        )
        return ExecutionResult(**response)

    def get_reconciliation_rule_result(self, execution_id) -> RuleResult:
        response = self._get(
            url=f'{self._catalog_api_base}/rules/reconciliation/executions/{execution_id}/result'
        )
        return ReconcillationExecutionResult(**response)

    def get_dq_rule_result(self, execution_id) -> RuleResult:
        response = self._get(
            url=f'{self._catalog_api_base}/rules/data-quality/executions/{execution_id}/result'
        )
        return DataQualityExecutionResult(**response)

    def policy_executions(self, identifier, rule_type: RuleType, page=0, size=25,
                          sortBy='finishedAt:DESC') -> List[RuleExecutionSummary]:
        params = dict()
        params['page'] = page
        params['size'] = size
        params['sortBy'] = sortBy
        rule_id = identifier
        if not check_int(identifier):
            if rule_type.value == RuleType.DATA_QUALITY.value:
                rule_res = self.get_dq_rule(identifier)
            elif rule_type.value == RuleType.RECONCILIATION.value:
                rule_res = self.get_reconciliation_rule(identifier)
            rule_id = rule_res.rule.id

        response = self._get(
            url=f'{self._catalog_api_base}/rules/{rule_type.value.lower()}/{rule_id}/executions', params=params
        )
        executions = list(response['executions'])
        rule_execution_list = []
        for res in executions:
            execution = res['execution']
            rule_execution_list.append(RuleExecutionSummary(**execution))
        return rule_execution_list

    def get_all_rules(self, filter: PolicyFilter, page=0, size=25, withLatestExecution=True,
                               sortBy='updatedAt:DESC') -> RuleResult:
        params = dict()
        params['page'] = page
        params['size'] = size
        params['withLatestExecution'] = withLatestExecution
        params['sortBy'] = sortBy
        if filter.policyType is not None:
            params['ruleType'] = filter.policyType.name
            if filter.policyType.name == RuleType.RECONCILIATION.name:
                params['ruleType'] = params['ruleType'] + ',EQUALITY'
        if filter.lastExecutionResult is not None:
            params['resultStatus'] = filter.lastExecutionResult.name
        if filter.enable is not None:
            params['enabled'] = filter.enable
        if filter.active is not None and filter.active is True:
            params['ruleStatus'] = 'ACTIVE'
        else:
            params['ruleStatus'] = 'ALL'
        if filter.assets is not None:
            params['assetIds'] = ",".join([str(i.id) for i in filter.assets])
        if filter.data_sources is not None:
            params['assemblyIds'] = ",".join([str(i.id) for i in filter.data_sources])
        if filter.period is not None:
            finished_before_time = datetime.now()
            if filter.period == ExecutionPeriod.Last15minutes:
                started_after_time = datetime.now() - timedelta(minutes = 15)
            elif filter.period == ExecutionPeriod.Last30minutes:
                started_after_time = datetime.now() - timedelta(minutes = 30)
            elif filter.period == ExecutionPeriod.Last1hour:
                started_after_time = datetime.now() - timedelta(hours = 1)
            elif filter.period == ExecutionPeriod.Last3hours:
                started_after_time = datetime.now() - timedelta(hours = 3)
            elif filter.period == ExecutionPeriod.Last6hours:
                started_after_time = datetime.now() - timedelta(hours = 6)
            elif filter.period == ExecutionPeriod.Last12hours:
                started_after_time = datetime.now() - timedelta(hours = 12)
            elif filter.period == ExecutionPeriod.Last24hours:
                started_after_time = datetime.now() - timedelta(hours = 24)
            elif filter.period == ExecutionPeriod.Today:
                started_after_time = datetime.now() - timedelta(hours = 24)
            elif filter.period == ExecutionPeriod.Yesterday:
                started_after_time = datetime.now() - timedelta(days = 2)
                finished_before_time = datetime.now() - timedelta(days = 1)
            elif filter.period == ExecutionPeriod.Last7days:
                started_after_time = datetime.now() - timedelta(days = 7)
            elif filter.period == ExecutionPeriod.Thismonth:
                started_after_time = datetime.now() - timedelta(days = 30)
            elif filter.period == ExecutionPeriod.Last1month:
                started_after_time = datetime.now() - timedelta(days = 30)
            elif filter.period == ExecutionPeriod.Last3month:
                started_after_time = datetime.now() - timedelta(days = 90)

            params['finishedBefore'] = int(round(finished_before_time.timestamp() * 1000))
            params['startedAfter'] = int(round(started_after_time.timestamp() * 1000))

        response = self._get(
            url=f'{self._catalog_api_base}/rules',
            params=params
        )
        rules = list(response['rules'])
        rule_list = []
        for res in rules:
            rule = res['rule']
            rule_list.append(DataQualityRule(**rule))
        return rule_list

    @staticmethod
    def now_ms():
        return int(round(time.time() * 1000))

    def _post(self, url, payload=None, as_json=True):
        now_ms = self.now_ms()
        if _HEADERS['accessKey'] is None:
            _HEADERS['accessKey'] = self.access_key
        if _HEADERS['secretKey'] is None:
            _HEADERS['secretKey'] = self.secret_key
        response = requests.post(
            url=url, headers=_HEADERS, json=payload, timeout=self._timeout, verify=False
        )
        self.logger.info(
            f" POST {url} "
            f"payload={json.dumps(payload)} "
            f"duration_ms={self.now_ms() - now_ms}"
        )

        return self._response(response, as_json)

    def _put(self, url, payload=None, as_json=True):
        now_ms = self.now_ms()
        if _HEADERS['accessKey'] is None:
            _HEADERS['accessKey'] = self.access_key
        if _HEADERS['secretKey'] is None:
            _HEADERS['secretKey'] = self.secret_key
        response = requests.put(
            url=url, headers=_HEADERS, json=payload, timeout=self._timeout, verify=False
        )
        self.logger.info(
            f" PUT {url} "
            f"payload={json.dumps(payload)} "
            f"duration_ms={self.now_ms() - now_ms}"
        )

        return self._response(response, as_json)

    def _get(self, url, params=None, as_json=True):
        now_ms = self.now_ms()
        if _HEADERS['accessKey'] is None:
            _HEADERS['accessKey'] = self.access_key
        if _HEADERS['secretKey'] is None:
            _HEADERS['secretKey'] = self.secret_key
        response = requests.get(
            url=url, params=params, headers=_HEADERS, timeout=self._timeout, verify=False
        )
        self.logger.info(
            f" GET {url} "
            f"duration_ms={self.now_ms() - now_ms}"
        )

        return self._response(response, as_json)

    def _post_form_data(self, url, payload=None, as_json=True):
        now_ms = self.now_ms()
        if _HEADERS['accessKey'] is None:
            _HEADERS['accessKey'] = self.access_key
        if _HEADERS['secretKey'] is None:
            _HEADERS['secretKey'] = self.secret_key

        params = {
            'request': json.dumps(payload)
        }
        data = MultipartEncoder(fields=params)
        _HEADERS['Content-Type'] = data.content_type
        response = requests.post(
            url=url, headers=_HEADERS, data=data, timeout=self._timeout, verify=False
        )
        self.logger.info(
            f" POST {url} "
            f"payload={json.dumps(payload)} "
            f"duration_ms={self.now_ms() - now_ms}"
        )

        return self._response(response, as_json)

    def _put_form_data(self, url, payload=None, as_json=True):
        now_ms = self.now_ms()
        if _HEADERS['accessKey'] is None:
            _HEADERS['accessKey'] = self.access_key
        if _HEADERS['secretKey'] is None:
            _HEADERS['secretKey'] = self.secret_key

        params = {
            'request': json.dumps(payload)
        }
        data = MultipartEncoder(fields=params)
        _HEADERS['Content-Type'] = data.content_type
        response = requests.put(
            url=url, headers=_HEADERS, data=data, timeout=self._timeout, verify=False
        )
        self.logger.info(
            f" POSPUTT {url} "
            f"payload={json.dumps(payload)} "
            f"duration_ms={self.now_ms() - now_ms}"
        )

        return self._response(response, as_json)

    def _delete(self, url, params=None, as_json=True):
        now_ms = self.now_ms()
        if _HEADERS['accessKey'] is None:
            _HEADERS['accessKey'] = self.access_key
        if _HEADERS['secretKey'] is None:
            _HEADERS['secretKey'] = self.secret_key
        response = requests.delete(
            url=url, params=params, headers=_HEADERS, timeout=self._timeout, verify=False
        )
        self.logger.info(
            f" DELETE {url} "
            f"duration_ms={self.now_ms() - now_ms}"
        )

        return response

    def _response(self, response, as_json):
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            self.logger.error(f' {response.text}')
            self._raise_api_error(e, response.text)

        return response.json() if as_json else response.text

    def _raise_api_error(self, e, text):
        raise APIError(text) from e



from acceldata_sdk.torch_client import TorchClient
import acceldata_sdk.constants as const

from acceldata_sdk.models.profile import ProfilingType
from acceldata_sdk.models.profile import AutoProfileConfiguration, Profile, ProfileRequest, ProfilingType
import pprint

pp = pprint.PrettyPrinter(indent=4)
# pipeline_uid = "torch.airflow.demo.lambda"
# torch_credentials = {
#     'url': 'https://torchdemo.acceldata.tech/torch',
#     'access_key': 'V05DG18TV4MAF93',
#     'secret_key': 'VZF8O9Q5G6GUE08MQY5A8CA0TAE5KD'
# }
# torch_credentials = {
#     'url': 'https://torch.acceldata.local:5443/torch',
#     'access_key': 'P04IM8FNQRUCRTU',
#     'secret_key': 'E6LL9YUPMG4BDTJHT2VZD75HW0B8E5',
#     'do_version_check': False
# }


# torch_credentials = {
#     'url': 'https://yubi.torch.acceldata.dev/torch',
#     'access_key': 'SAQ8XAOLSSLAD16',
#     'secret_key': 'KL5C4UA5XBQ2HBUMQPATJ3FB45HRH9'
# }



torch_credentials = {
    'url': 'https://torch.acceldata.local:5443/torch',
    'access_key': 'P04IM8FNQRUCRTU',
    'secret_key': 'E6LL9YUPMG4BDTJHT2VZD75HW0B8E5'
}


torch_client = TorchClient(**torch_credentials)
# pipelines = torch_client.get_pipelines()
# print (f"number of pipelines: {len(pipelines)}")
# for pipeline in pipelines:
#     print(f"pipeline id: {pipeline.pipeline.id}")
#     if pipeline.pipeline.id > 77:
#         print (f"pipeline id being deleted: {pipeline.pipeline.id}")
#         pipeline.pipeline.delete()
i = 77
while i<150:
    try:
        pipeline = torch_client.get_pipeline(i)
        if pipeline is not None:
            print(f'deleting {i}')
            pipeline.delete()
    except:
        print (f'{i} not found')
    i = i+1




exit()
pipeline_uid = "torch.test"


pipeline = torch_client.get_pipeline(pipeline_uid)
pipeline_run = pipeline.create_pipeline_run()
run_with_id = pipeline.get_run(pipeline_run.id)
exit()

# recon_rule_id=4
# import pdb;pdb.set_trace()
# recon_rule2 = torch_client.execute_rule(const.RECONCILIATION, recon_rule_id, sync=True, incremental=False, pipeline_run_id=401)
# sync_execution_result = recon_rule2.get_result()
# exit(0)


#### pipelines ###

torch_client = TorchClient(**torch_credentials)
supported_versions = torch_client.get_supported_sdk_versions()
pp.pprint('supported_versions')
pp.pprint(supported_versions)





pipeline = torch_client.get_pipeline(42)
pp.pprint('pipeline')
pp.pprint(pipeline)
runs = pipeline.get_runs()
pp.pprint('runs')
pp.pprint(runs)
pipeline_uid = torch_client.get_pipeline('torch.airflow.demo.lambda')
pp.pprint('pipeline_uid')
pp.pprint(pipeline_uid)
pipelines = torch_client.get_pipelines()
pp.pprint('pipelines')
pp.pprint(pipelines)
pipelineRun = torch_client.get_pipeline_run(244)
pp.pprint('pipelineRun')
pp.pprint(pipelineRun)
pipeline_details = pipelineRun.get_details()
pp.pprint('pipeline_details')
pp.pprint(pipeline_details)
pipeline_spans = pipelineRun.get_spans()
pp.pprint('pipeline_spans')
pp.pprint(pipeline_spans)
pipeline = torch_client.get_pipeline(33)
# delete_response = pipeline.delete()
#### pipelines ###
# import pdb;pdb.set_trace()
# async_executor = torch_client.execute_policy(const.PolicyType.DATA_QUALITY, 3, sync=True)
# xcom_key = f'{const.PolicyType.DATA_QUALITY.name}_{3}_execution_id'
# pp.pprint(xcom_key)
# if async_executor.errorMessage is not None:
#     async_execution_result = async_executor.get_result()

###suman test###



torch_credentials = {
    'url': 'https://torch.acceldata.local:5443/torch',
    'access_key': 'P04IM8FNQRUCRTU',
    'secret_key': 'E6LL9YUPMG4BDTJHT2VZD75HW0B8E5'
}
torchclient = TorchClient(**torch_credentials)
# print(torchclient.get_datasource(assemblyName, True))
print(torchclient.get_datasource(1, True))

print(torchclient.get_datasource('sf_ds', True))
print(torchclient.get_datasources(const.AssetSourceType.SNOWFLAKE))
torch_credentials = {
    'url': "https://testtorchswetha.qetest.acceldata.dev",
    'access_key': '95UZMC25GGPU2HD',
    'secret_key': 'ZHRK78X34GH5JBNU4TXYHU2E2AOBCY'
}
torchclient = TorchClient(**torch_credentials)
# print(torchclient.get_datasource(assemblyName, True))
print(torchclient.get_datasource(1, True))

print(torchclient.get_datasource('sf_ds', False))
print(torchclient.get_datasources(const.AssetSourceType.SNOWFLAKE))
exit()
#####

#
# torch_credentials = {
#     'url': 'https://test.torch1001.acceldata.dev/torch',
#     'access_key': 'WKZW28UJGPHVJZJ',
#     'secret_key': '5XP4HKAK0PBP00UQOUF89FEWDO7ZB6'
# }


'''dss = torch_client.get_datasources()
pp.pprint('datasources')
pp.pprint(dss)
dss = torch_client.get_datasources(const.AssetSourceType.SNOWFLAKE)
pp.pprint('datasources snowflake')
pp.pprint(dss)

ds = torch_client.get_datasource(1, True)
pp.pprint('datasource with properties')
pp.pprint(ds)
asset = ds.get_asset(251)
pp.pprint('asset')
pp.pprint(asset)
response_start_crawler = ds.start_crawler()
pp.pprint('response_start_crawler')
pp.pprint(response_start_crawler)
response_status_crawler = ds.get_crawler_status()
pp.pprint('response_status_crawler')
pp.pprint(response_status_crawler)'''

# asset = torch_client.get_asset(251)
# pp.pprint('asset')
# pp.pprint(asset)
'''metadata_asset = asset.get_metadata()
pp.pprint('metadata_asset')
pp.pprint(metadata_asset)
sample_asset = asset.sample_data()
pp.pprint('sample_asset')
pp.pprint(sample_asset)

asset.add_labels(labels=[AssetLabel('test1', 'shubh1'), AssetLabel('test2', 'shubh3')])
labels_asset = asset.get_labels()
pp.pprint('labels_asset')
pp.pprint(labels_asset)
asset.add_custom_metadata(custom_metadata=[CustomAssetMetadata('testcm1', 'shubhcm1'), CustomAssetMetadata('testcm2', 'shubhcm2')])
latest_profile_status_asset = asset.get_latest_profile_status()
pp.pprint('latest_profile_status_asset')
pp.pprint(latest_profile_status_asset)'''
start_profile_asset = asset.start_profile(ProfilingType.FULL)
pp.pprint('start_profile_asset')
pp.pprint(start_profile_asset)
profile_status = start_profile_asset.get_status()
pp.pprint('profile_status')
pp.pprint(profile_status)
cancel_res = profile_status = start_profile_asset.cancel()
pp.pprint('cancel_res')
pp.pprint(cancel_res)



# asset2 = ds.get_asset(1558)
# dq_rule = torch_client.get_policy(const.PolicyType.RECONCILIATION, "auth001_reconciliation")
from acceldata_sdk.models.ruleExecutionResult import RuleType, PolicyFilter, ExecutionPeriod
# filter = PolicyFilter(policyType=RuleType.RECONCILIATION, enable=True)
dq_rule_execution = torch_client.get_all_rule_execution(RuleType.DATA_QUALITY)
pp.pprint(dq_rule_execution)
filter = PolicyFilter(policyType=RuleType.DATA_QUALITY, enable=True)
dq_rules = torch_client.list_all_policies(filter=filter)
pp.pprint(dq_rules)
# asset = ds.get_asset(1207)
# # asset.add_labels(labels=[AssetLabel('test1', 'shubh1'), AssetLabel('test2', 'shubh3')])
# asset.add_custom_metadata(custom_metadata=[CustomAssetMetadata('testcm1', 'shubhcm1'), CustomAssetMetadata('testcm2', 'shubhcm2')])

# pipeline_uid = "5321.airflow.coke.precreate13594"

# torch_client = TorchClient(**torch_credentials)
# pipeline = torch_client.get_pipeline(pipeline_uid)
# pipeline_run = pipeline.get_latest_pipeline_run()
# span_context = pipeline_run.get_root_span()

# async_executor = torch_client.execute_policy(const.DATA_QUALITY, 46, sync=False)
# async_execution_result = async_executor.get_result()
# import pdb;pdb.set_trace()
# execution_current_status = async_executor.get_status()
# pp.pprint("async_execution_result")
# pp.pprint(async_execution_result)
# #
# sync_executor_result = torch_client.execute_policy(const.DATA_QUALITY, 46, sync=True)
# import pdb;pdb.set_trace()
# #pp.pprint(sync_executor_result)
#
# execution_status = sync_executor_result.get_status(execution_details.id)
# sync_execution_result = sync_executor_result.get_result()
# pp.pprint("sync_execution_result")
# pp.pprint(sync_execution_result)

# torch_client = TorchClient(url='https://demo.acceldata.app', access_key='IV7QOLGIPBUH4M9', secret_key='2FYBOHEU0GDPCFOYIJVE7XK6FWF4IJ')
#
# recon_rule_id=132
# import pdb;pdb.set_trace()
# recon_rule2 = torch_client.execute_policy(const.RECONCILIATION, recon_rule_id, sync=True, incremental=False)
# sync_execution_result = recon_rule2.get_result()




import logging

from acceldata_sdk.initialiser import torch_client_credentials
from acceldata_sdk.models.pipeline import CreatePipeline, PipelineMetadata
from acceldata_sdk.torch_client import TorchClient

logging.basicConfig(level=logging.INFO)

# Set up acceldata_sdk_code client
torch_client = TorchClient(url="https://torch.acceldata.local:5443",
                       access_key="N1LTYRK630PZ", secret_key="xPeUj4Iyj4WL2Tw284s9mqsgxvbPKW")
# create pipeline object
# pipeline = CreatePipeline(
#     uid='monthly_reporting-airflow_span_decorator',
#     name='Monthly reporting Pipeline-airflow span_decorator',
#     description='Pipeline to create monthly reporting tables',
#     meta=PipelineMetadata('Vaishvik', 'acceldata_sdk_code', '...'),
#     context={'key1': 'value1'}
# )
pipeline = CreatePipeline(
    uid='customer.orders.monthly.agg.demo',
    name='Customer Orders Monthly aggregate',
    description='Pipeline to Aggregate the customer orders over 1 year',
    meta=PipelineMetadata(
        owner='vaishvik', team='acceldata_sdk_code', codeLocation='...'),
    context={
        'associated_tables': 'pipeline.customer, pipeline.orders, pipeline.customer_orders, pipeline.customer_orders_monthly_agg'}
)
# creating pipeline using acceldata_sdk_code client
pipelineResponse = torch_client.create_pipeline(pipeline=pipeline)
print('Newly Created Pipeline Response : ', pipelineResponse)

# create a pipeline run of the pipeline
pipelineRunResponse = pipelineResponse.create_pipeline_run(context_data={'team': 'acceldata_sdk_code', 'name': 'backend'})
print('pipeline Run Created :', pipelineRunResponse)
#
# create span in the pipeline run
spanContext = pipelineRunResponse.create_span(uid='main_span',
                                              context_data={'start': 'tuesday', 'name': 'vaishvik'})
print('Span Created :', spanContext)

# check current span is root or not
print('span is root or not : ', spanContext.is_root())

# end the span event
print('Span End Event Response :', spanContext.end())

# check if the current span has children or not
print('has child ', spanContext.has_children())

# create a child span
childSpanContext = spanContext.create_child_span('span_uid_39_2')
print('child span is root or not  ', childSpanContext.is_root())
print('child span abort event : ', childSpanContext.abort())

print('child span has child or not : ', childSpanContext.has_children())
print('parent span has child or not : ', spanContext.has_children())



from time import sleep

from models.connection import CreateConnection
from acceldata_sdk.models.profile import ProfilingType, JobType
from models.tags import ClassificationConfig
from acceldata_sdk.models.create_asset import AssetMetadata
from torch_client import TorchClient
from acceldata_sdk.models.datasource import CreateDataSource, DatasourceType, ConfigProperty

torch_client = TorchClient(url="https://torch.acceldata.local:5443",
                           access_key="N1LTYRK630PZ", secret_key="xPeUj4Iyj4WL2Tw284s9mqsgxvbPKW")
from acceldata_airflow_sdk.decorators.span import span
from acceldata_airflow_sdk.operators.span_operator import SpanOperator


# get connection types
# connection_types = torch_client.get_connection_types()
# print(connection_types)

# get tags defined in torch, update protection flag, delete tag
# tags = torch_client.get_tags()
# print(tags)
# torch_client.update_data_protection(tag='asset_tag', is_protected=True)
# torch_client.delete_tag(tag='asset_tag')


# get related assets and child assets
# datasource = torch_client.get_datasource('postgres_ds_local')
# asset = datasource.get_asset(id=20)
# relations = asset.get_related_assets()
# print(relations)
# child_assets = asset.get_child_assets()
# print(child_assets)

# discover
# discover_res = torch_client.discover()
# print(discover_res)

# watch asset
# start_watch = asset.start_watch()
# print(start_watch)
# asset.stop_watch()

# tags
# tags = asset.get_asset_tags()
# print(tags)
# tag_add = asset.add_asset_tag(tag='asset_tag')
# print(tag_add)
# tag_rm = asset.remove_asset_tag(tag='asset_tag')
# print(tag_rm)


# asset activty
# activty = asset.get_asset_activity()
# print(activty)

# asset comment
# comments = asset.get_asset_comment()
# print(comments)

# labels
# labels = asset.add_asset_labels(labels = ['teamtorch', 'comp:adinc'])
# print(labels)
# labels = asset.get_asset_labels()
# print(labels)

# annotation
# annotation = asset.update_asset_annotation(annotation='this asset is created and accessed from acceldata sdk')
# print(annotation)

# delete an asset
# datasource.delete_asset(id=9523)

# list all snapshots
# snapshots = datasource.list_all_snapshots()
# print(snapshots)
# snapshots = torch_client.list_all_snapshots()
# print(snapshots)

# get user
# user = torch_client.get_user(user_id=3)
# print(user)
# user = torch_client.get_user(username='admin')
# print(user)

# crawler
# datasource.start_crawler()
# datasource.get_crawler_status()
# datasource.restart_crawler()

# api keys
# keys = torch_client.create_api_key()
# print(torch_client.get_api_keys())
# torch_client.delete_api_key(keys.accessKey)

# notifications
# print(torch_client.get_notifications())
# print(torch_client.get_incidents())

# analysis pipelines
# print(torch_client.get_analysis_pipeline(1))
# print(torch_client.list_all_analysis_pipelines())

# data sources : create, update, delete
# print(torch_client.get_all_datasources())

# datasource = CreateDataSource(
#     name='snowflake_ds_local',
#     sourceType=DatasourceType(5, 'SNOWFLAKE'),
#     description='snowflake schema',
#     connectionId=9,
#     configProperties=[ConfigProperty(key='jdbc.warehouse', value='COMPUTE_WH'),
#                       ConfigProperty(key='databases.0', value='FINANCE')]
# )
# ds_res = torch_client.create_datasource(datasource)
# ds_res = torch_client.get_datasource('snowflake_ds_local')

# datasource = CreateDataSource(
#     name='snowflake_ds_local',
#     sourceType=DatasourceType(5, 'SNOWFLAKE'),
#     description='snowflake schema',
#     connectionId=9,
#     configProperties=[ConfigProperty(key='jdbc.warehouse', value='COMPUTE_WH'),
#                       ConfigProperty(key='databases.0', value='CRAWLER_DB1')]
# )
# ds_res = ds_res.update_datasource(datasource)

# print(ds_res.get_root_assets())

# auto profile configs
# print(torch_client.get_all_auto_profile_configurations())
# ds_res = torch_client.get_datasource('postgres_ds_local')
# print(ds_res.get_auto_profile_configuration())
# ds_res.remove_auto_profile_configuration()

# asset metadata
# asset = torch_client.get_asset(id=1128)
# metadata = [AssetMetadata('STRING', 'key', 'source', 'value'), AssetMetadata('STRING', 's3_location', 'AWS_S3', 's3://aws/path/test/logs')]
# asset.update_asset_metadata(metadata)
# print(asset.get_metadata())

# profile an asset, get profile req details, cancel profile, autotag profile
# asset = torch_client.get_asset(id=3)
# profile_res = asset.profile_asset(profiling_type= ProfilingType.SAMPLE, job_type=JobType.MINI_PROFILE)
# sleep(10)
# profile_req_details = profile_res.get_status()
# cancel_profile_res = profile_res.cancel()
# profile_res = asset.get_profile_status()
# autotag_res = asset.auto_tag_asset()
# profile_req_details_by_req_id = asset.get_status(req_id= 845)

# sample data
# sample_data = asset.sample_data()

# classification configs : get all, create, update, delete
# configs = torch_client.list_all_classification_configurations()
# create_config = ClassificationConfig(
#     name='CCV',
#     protectedData=False,
#     classification='regex',
#     classificationType='card',
#     enabled= False,
#     description='Card CCV Checking with regex',
#     defaultValue='^\\d{3}$',
#     value='^\\d{4}$'
# )
# config_created = torch_client.create_classification_configuration(create_config)
# config = torch_client.get_classification_configuration(name='CCV')
# update_config = ClassificationConfig(
#     id=config.id,
#     name='CCV',
#     protectedData=False,
#     classification='regex',
#     classificationType='card',
#     enabled= False,
#     description='Card CCV Checking with regex',
#     defaultValue='^\\d{3}$',
#     value='^\\d{3}$'
# )
# config_updated = torch_client.update_classification_configuration(update_config)
# torch_client.delete_classification_configuration(name='CCV')


# setting groups
# setting_groups = torch_client.list_setting_groups()
# settings = torch_client.find_settings_by_group(setting_group_name= 'data_protection')
# setting = torch_client.get_setting(key='notification.channels.email.toEmail')
# updated_setting = torch_client.update_setting_value(key='data.protection.enabled', value="true")
# reset_setting_value = torch_client.reset_setting_value(key='data.protection.enabled')

# connections : create, update, check, list all
# connection_list = torch_client.list_all_connections()
# print(connection_list)
# torch_client.delete_connection(id=12)
# create_conn_obj = CreateConnection(
#     name='mysql_connection_sdk',
#     description='create connection from acceldata sdk',
#     connection_type_id=12,
#     analytics_pipeline_id=1,
#     properties=[ConfigProperty(key='jdbc.url', value='jdbc:postgresql://localhost:5432/'),
#                 ConfigProperty(key='jdbc.user', value='admin'),
#                 ConfigProperty(key='jdbc.password', value='admin')]
# )
# created_conn = torch_client.create_connection(create_conn_obj)
# update_conn_obj = CreateConnection(
#     name='mysql_connection_sdk',
#     description='create connection from acceldata sdk',
#     connection_type_id=12,
#     analytics_pipeline_id=1,
#     properties=[ConfigProperty(key='jdbc.url', value='jdbc:postgresql://localhost:5432/'),
#                 ConfigProperty(key='jdbc.user', value='admin'),
#                 ConfigProperty(key='jdbc.password', value='admin')]
# )
# updated_conn = torch_client.update_connection(update_connection=update_conn_obj)
# check_conn_status = torch_client.check_connection(update_conn_obj)
# get_conn = torch_client.get_connection(name='mysql_connection_sdk')


# rule execution and ops
# enable_rule = torch_client.enable_rule(rule_id=12)
# disable_rule = torch_client.disable_rule(rule_id=18)
# cancel_rule = torch_client.cancel_rule_execution(execution_id=1)
# execute_dq_rule = torch_client.execute_dq_rule(rule_id=18, incremental=False)
# execute_reconciliation_rule = torch_client.execute_reconciliation_rule(rule_id=24, incremental=False)
get_dq_execution_details = torch_client.get_dq_rule_execution_details(execution_id=62)
print(get_dq_execution_details.execution.executionStatus)
print(get_dq_execution_details.execution.resultStatus)

# get_recon_exe_details = torch_client.get_reconciliation_rule_execution_details(execution_id=64)
# print(get_recon_exe_details)


import psycopg2
import random
import datetime
import time

from acceldata_sdk.models.job import CreateJob, JobMetadata, Dataset
from acceldata_sdk.events.generic_event import GenericEvent
from acceldata_sdk.models.pipeline import CreatePipeline, PipelineMetadata, PipelineRunResult, PipelineRunStatus
from acceldata_sdk.torch_client import TorchClient

# data_source_name = 'postgres_ds_local'
# asset_1 = 'ad_catalog.ad_catalog.flyway_schema_history'
# asset_2 = 'ad_catalog.ad_catalog.source_models'
# asset_3 = 'ad_catalog.ad_catalog.source_types'
# asset_4 = 'ad_catalog.ad_catalog.assets'

# data_source_name = 'SNOWFLAKE_DS_PUBLIC'
# asset_1 = 'FINANCE.FINANCE.A1'
# asset_2 = 'FINANCE.FINANCE.A2'
# asset_3 = 'FINANCE.FINANCE.A3'
# asset_4 = 'FINANCE.FINANCE.A4'

data_source_name = 'REDSHIFT_DS'
asset_1 = 'dev.customers.test'
asset_2 = 'dev.customers.test2'
asset_3 = 'dev.customers.contacts'
asset_4 = 'dev.customers.datatypes'

def create_torch_client():
    # return TorchClient(
    #     url='https://neptune.demo.acceldata.dev/torch',
    #     access_key='VK8ZHTKZOMK2WH4',
    #     secret_key='G0HQVA47GOBWJ16RHLFU0D5V82AC2F')

    return TorchClient(
        url='https://winterfell.demo.acceldata.dev/torch',
        access_key='OG1YBHVDB84H11L',
        secret_key='EP4D1D2DKZSXXHCZLY5Z2YVQKA81FB')

    # return TorchClient(
    #     url='https://acceldata.acceldata.local:5443/torch',
    #     access_key='XO3JFEI02G93XS0',
    #     secret_key='1HFHZ9JMQQCHBD9DS3742B7TMDA0S8')
    #
    # return TorchClient(
    #     url='https://pepsi.acceldata.local:5443/torch',
    #     access_key='K9ZMZH2HBF8DZYV',
    #     secret_key='RITCFPMGU0AO2REZP05PRCF0RQBPQF')


def create_pipeline(torch_client):
    pipeline = CreatePipeline(
        uid='sso.multitenancy.etl.testing',
        name='SSO Multi tenancy testing - acceldata',
        description='Pipeline to Aggregate the customer orders over 1 year',
        meta=PipelineMetadata(
            owner='vaishvik', team='acceldata_sdk_code', codeLocation='...'),
        context={
            'associated_tables': 'pipeline.customer, pipeline.orders, pipeline.customer_orders, '
                                 'pipeline.customer_orders_monthly_agg'}
    )
    pipeline_response = torch_client.create_pipeline(pipeline=pipeline)
    print('Created the pipeline')
    return pipeline_response


def create_datagen_job(pipeline):
    pipeline_run = pipeline.get_latest_pipeline_run()
    job = CreateJob(
        uid='customer.order.datagen.job',
        name='customers and orders datagen',
        version=pipeline_run.versionId,
        description='Generates Pseudo random data for Orders and Customers',
        inputs=[],
        outputs=[
            Dataset(data_source_name,
                    asset_1), Dataset(data_source_name,
                                      asset_2)],
        meta=JobMetadata(owner='vaishvik', team='backend',
                         codeLocation='https://github.com/acme/reporting/reporting.scala'),
        context={}
    )
    job = pipeline.create_job(job)
    print('Created Job for random data insertion')
    return job


def create_data_join_job(pipeline):
    pipeline_run = pipeline.get_latest_pipeline_run()
    job = CreateJob(
        uid='customer.order.join.job',
        name='customers and orders joiner and Inserter',
        version=pipeline_run.versionId,
        description='Joins the ',
        inputs=[Dataset(data_source_name, asset_1), Dataset(data_source_name, asset_2)],
        outputs=[
            Dataset(data_source_name,
                    asset_3)],
        meta=JobMetadata(owner='vaishvik', team='backend',
                         codeLocation='https://github.com/acme/reporting/reporting.scala'),
        context={}
    )
    job = pipeline.create_job(job)
    print('Created Job for joined data read and insert')
    return job


def create_pipeline_run(pipeline):
    return pipeline.create_pipeline_run(
        context_data={'client_time': str(datetime.datetime.now())})


def start_main_span(pipeline_run):
    span_context = pipeline_run.create_span(
        uid='customer.orders.monthly.agg', context_data={'client_time': str(datetime.datetime.now())})
    return span_context


def end_main_span(span_context):
    span_context.end()


def end_pipeline_run(pipeline_run, result=PipelineRunResult.SUCCESS, status=PipelineRunStatus.COMPLETED):
    pipeline_run.update_pipeline_run(context_data={'client_time': str(datetime.datetime.now())},
                                     result=result,
                                     status=status)


def create_data_agg_job(pipeline):
    pipeline_run = pipeline.get_latest_pipeline_run()
    job = CreateJob(
        uid='monthly.order.aggregate.job',
        name='Aggregates the monthly orders',
        version=pipeline_run.versionId,
        description='Joins the ',
        inputs=[Dataset(data_source_name,
                        asset_3)],
        outputs=[
            Dataset(data_source_name,
                    asset_4)],
        meta=JobMetadata(owner='vaishvik', team='backend',
                         codeLocation='https://github.com/acme/reporting/reporting.scala'),
        context={}
    )
    job = pipeline.create_job(job)
    print('Created Job for monthly aggregation')
    return job


def insert_data(span_context):
    datagen_span_context = span_context.create_child_span(
        uid="customer.orders.datagen.span", context_data={'client_time': str(datetime.datetime.now())})
    customer_datagen_span = datagen_span_context.create_child_span(
        uid="customer.data.gen", context_data={'client_time': str(datetime.datetime.now())})

    time.sleep(2)
    customer_datagen_span.send_event(GenericEvent(context_data={'client_time': str(
        datetime.datetime.now()), 'customers_count': 100}, event_uid="customer.insert.data.result"))
    customer_datagen_span.end(
        {'client_time': str(datetime.datetime.now()), 'customers_count': 100})

    print("Creating dummy orders for each customers")
    orders_datagen_span = datagen_span_context.create_child_span(
        uid="order.data.gen", context_data={'client_time': str(datetime.datetime.now())})

    time.sleep(2)
    orders_datagen_span.send_event(GenericEvent(context_data={'client_time': str(
        datetime.datetime.now()), 'orders_count': 100}, event_uid="order.insert.data.result"))
    orders_datagen_span.end(
        {'client_time': str(datetime.datetime.now()), 'orders_count': 100})
    datagen_span_context.end({'client_time': str(datetime.datetime.now())})


def execute_join_orders_query(span_context):
    join_span_context = span_context.create_child_span(
        uid="customer.orders.join.span", context_data={'client_time': str(datetime.datetime.now())})
    print("Joining customers and orders")

    time.sleep(2)
    join_span_context.send_event(GenericEvent(context_data={'client_time': str(
        datetime.datetime.now()), 'row_count': 100}, event_uid="order.customer.join.result"))
    join_span_context.end(
        {'client_time': str(datetime.datetime.now()), 'row_count': 100})


def insert_orders(span_context):
    print("Inserting joined record into the orders table")
    insert_span_context = span_context.create_child_span(
        uid="customer.orders.insert.span", context_data={'client_time': str(datetime.datetime.now())})
    time.sleep(2)
    insert_span_context.send_event(GenericEvent(context_data={'client_time': str(
        datetime.datetime.now()), 'order_placed': 42}, event_uid="orders.placed.insert.data.result"))
    insert_span_context.end({'client_time': str(datetime.datetime.now())})


def insert_aggregate_data(span_context):
    print("Aggregating the orders over months per customer")
    agg_span_context = span_context.create_child_span(
        uid="monthy.aggregate.span", context_data={'client_time': str(datetime.datetime.now())})
    time.sleep(2)
    agg_span_context.send_event(GenericEvent(context_data={'client_time': str(
        datetime.datetime.now()), 'agg_data_counts': 62}, event_uid="monthly.aggregate.result"))
    agg_span_context.end({'client_time': str(datetime.datetime.now())})


if __name__ == "__main__":
    torch_client = create_torch_client()

    pipeline = create_pipeline(torch_client)
    pipeline_run = create_pipeline_run(pipeline)
    create_datagen_job(pipeline)
    create_data_join_job(pipeline)
    create_data_agg_job(pipeline)

    span_context = start_main_span(pipeline_run)
    insert_data(span_context)
    time.sleep(2)
    execute_join_orders_query(span_context)
    time.sleep(2)
    insert_orders(span_context)
    time.sleep(2)
    insert_aggregate_data(span_context)
    time.sleep(2)
    end_main_span(span_context)
    end_pipeline_run(pipeline_run)



from dataclasses import dataclass, asdict
from enum import Enum
from typing import List

from acceldata_sdk.models.job import CreateJob
from acceldata_sdk.models.span import Span
from acceldata_sdk.models.span_context import SpanContext
from datetime import datetime


@dataclass
class Tag:
    id = None
    name = None
    displayName = None
    createdAt = None
    updatedAt = None


@dataclass
class PipelineMetadata:
    """
    Description:
        Pipeline metadata.

    :param owner : (String) owner of the pipeline
    :param team: (String) team of the owner
    :param codeLocation: (String) location of the code
    """
    owner: str = None
    team: str = None
    codeLocation: str = None


class CreatePipeline:
    # createdAt = None
    # enabled = None
    # id = None
    # interrupt = None
    # notificationChannels = None
    # schedule = None
    # scheduled = None
    # schedulerType = None
    # updatedAt = None

    def __init__(self, uid: str, name: str,
                 description: str = None, meta: PipelineMetadata = None, **context):
        """
        Description:
            Class used for create pipeline in torch catalog
        :param uid: uid of the pipeline
        :param name: name of the pipeline
        :param description: (Optional)description of the pipeline
        :param meta: (PipelineMetadata) meta data of the pipeline
        :param context: context map of the pipeline
        """
        self.uid = uid
        self.name = name
        self.description = description
        self.context = context
        if meta is not None:
            self.meta = PipelineMetadata(owner=meta.owner, team=meta.team, codeLocation=meta.codeLocation)
        else:
            self.meta = None

    def __eq__(self, other):
        return self.uid == other.uid

    def __repr__(self):
        return f"Pipeline({self.uid!r})"

    def add_context(self, key: str, value: str):
        """
        Used to add context in context map
        :param key: key of the context
        :param value: value of the context
        :return: context dir
        """
        self.context[key] = value
        return self.context


class PipelineRunStatus(Enum):
    """
        pipeline run status
    """
    STARTED = 1
    COMPLETED = 2
    FAILED = 3
    ABORTED = 4


class PipelineRunResult(Enum):
    """
        Pipeline run result
    """
    RUNNING = 'RUNNING'
    SUCCESS = 'SUCCESS'
    FAILURE = 'FAILURE'


class NodeType(Enum):
    """
        Pipeline Node Type
    """
    ASSET = 'ASSET'
    FUNCTIONAL = 'FUNCTIONAL'


class NodeAddition(Enum):
    """
        Pipeline Node Addition
    """
    PIPELINE_CREATION = 'PIPELINE_CREATION'
    MANUAL_ADDITION = 'MANUAL_ADDITION'
    QUERY_LOG_ANALYSIS = 'QUERY_LOG_ANALYSIS'


class NodeStatus(Enum):
    """
        Pipeline Node Status
    """
    ACTIVE = 'ACTIVE'
    STALE = 'STALE'
    DELETED = 'DELETED'


class PipelineRun:

    def __init__(self, pipelineId, versionId=None, status=PipelineRunStatus.STARTED, id=None, startedAt=None,
                 finishedAt=None, result=None, client=None, args=None,
                 continuationId=None, successEvents=None, errorEvents=None, warningEvents=None, avgExecutionTime=None,
                 *argumentss, **kwargs):
        """
        Description:
                Pipeline run used for instantiate run of the pipeline. To create new pipeline run you need to pass pipelineId, version.
        :param pipelineId: pipleline id
        :param versionId:  pipeline run current version
        :param status: status of the pipeline run
        :param id: pipeline run id
        :param startedAt: starting time
        :param finishedAt: finish time
        :param args: additional args for pipeline run
        :param result: pipeline run result
        """
        self.pipelineId = pipelineId
        self.versionId = versionId
        self.status = status
        self.args = args
        self.result = result
        if continuationId is not None:
            self.continuationId = continuationId
        if successEvents is not None:
            self.successEvents = successEvents
        if errorEvents is not None:
            self.errorEvents = errorEvents
        if warningEvents is not None:
            self.warningEvents = warningEvents
        if avgExecutionTime is not None:
            self.avgExecutionTime = avgExecutionTime
        if id is not None:
            self.id = id
            self.startedAt = startedAt
            self.finishedAt = finishedAt
            self.client = client

    def __eq__(self, other):
        return self.id == other.id

    def __repr__(self):
        return f"PipelineRun({self.__dict__})"

    # convert job object to dict type
    def _convert_job_to_dict(self, job: CreateJob):
        """
        Description:
            Convert createJob class instance to dict type
        :param job: createJob class instance
        :return: dict form of createJob class instance
        """
        job_dict = job.__dict__
        if hasattr(job, 'meta'):
            meta = asdict(job.meta)
            job_dict['meta'] = meta
        input_dict = []
        for ds_in in job.inputs:
            input_dict.append(asdict(ds_in, dict_factory=lambda x: {k: v for (k, v) in x if v is not None}))
        output_dict = []
        for ds_out in job.outputs:
            output_dict.append(asdict(ds_out, dict_factory=lambda x: {k: v for (k, v) in x if v is not None}))
        job_dict['inputs'] = input_dict
        job_dict['outputs'] = output_dict
        job_dict['version'] = self.versionId
        return job_dict

    # create job for pipeline run
    def create_job(self, job: CreateJob):
        """
        Description:
            used to create job for any pipeline run
        :return: Job class instance of created job        """
        if job.uid is None or job.name is None:
            raise Exception('To create a job job uid and name are required')
        payload = self._convert_job_to_dict(job)
        res = self.client.create_job(payload, self.pipelineId)
        if job.bounded_by_span and job.span_uid is not None:
            associated_job_uids = [job.uid]
            span_uid_temp = job.span_uid
            # Get root span to create child span under
            parent_span_context = self.get_root_span()
            if parent_span_context is not None:
                # Create child span under root span
                span_context = parent_span_context.create_child_span(
                    uid=span_uid_temp,
                    context_data={
                        'time': str(datetime.now())
                    },
                    associatedJobUids=associated_job_uids
                )
            else:
                # Create root span under pipeline_run as root span doesn't exist
                self.create_span(
                    uid=span_uid_temp,
                    context_data={
                        'time': str(datetime.now())
                    },
                    associatedJobUids=associated_job_uids
                )
        else:
            raise Exception('To create a span span_uid is required')
        return res

    # convert pipeline run to dict type
    def _convert_pipeline_run_to_dict(self, pipeline_run=None):
        """
            Description:
                Convert PipelineRun class instance to dict type
            :param pipeline_run: PipelineRun class instance
            :return: dict form of PipelineRun class instance
        """
        payload = pipeline_run.__dict__
        payload['status'] = pipeline_run.status.name
        if pipeline_run.result is not None:
            payload['result'] = pipeline_run.result.name
        else:
            del payload['result']
        pipeline_run_payload = {'run': payload}
        return pipeline_run_payload

    # update run for a pipeline
    def update_pipeline_run(self, result: PipelineRunResult,
                            status: PipelineRunStatus = PipelineRunStatus.STARTED,
                            context_data: {} = None):
        """
        Description:
            used to update an existing pipeline run
        :return: updated pipelineRun class instance
        """
        update_pipeline_run = PipelineRun(
            pipelineId=self.pipelineId,
            versionId=self.versionId,
            result=result,
            args=context_data,
            status=status
        )
        payload = self._convert_pipeline_run_to_dict(update_pipeline_run)
        res = self.client.update_pipeline_run(self.id, payload)
        return res

    # convert span to dict
    def _convert_span_to_dict(self, span: Span, associatedJobUids):
        """
            Description:
                Convert Span class instance to dict type
            :param span: Span class instance
            :return: dict form of Span class instance
        """
        payload = span.__dict__
        payload['status'] = span.status.name
        span_payload = {'span': payload, 'associatedJobUids' : associatedJobUids }
        return span_payload

    # create span for pipeline run
    def create_span(self, uid: str, context_data: dict = None, associatedJobUids = None ):
        """
        Description:
            used to create span for any pipeline run
        :param associatedJobUids: list of string (job uids)
        :param context_data
        :param uid: span uid
        :return: SpanContext of the span
        """
        if uid is None:
            raise Exception('Span uid can not be None. You need to pass span uid to create span')
        create_span = Span(
            uid=uid,
            pipelineRunId=self.id
        )
        if associatedJobUids is None:
            associatedJobUids = []
        elif not isinstance(associatedJobUids, list):
            raise Exception('associatedJobUids should be a list')

        payload = self._convert_span_to_dict(create_span, associatedJobUids)
        res = self.client.create_span(self.id, payload)
        span_context = SpanContext(client=self.client, span=res, context_data= context_data, new_span = True)
        return span_context

    def get_span(self, span_uid: str):
        """
            Description:
                Get span of the pipeline run by uid
        :param span_uid: uid of the span
        :return: SpanContext instance of the input span uid
        """
        span = self.client.get_span(pipeline_run_id= self.id, uid=span_uid)
        span_context = SpanContext(client=self.client, span= span)
        return span_context

    def get_root_span(self):
        """
            Description:
                Get root span of the pipeline run
        :return: SpanContext instance of the root span
        """
        span_context = None
        span = self.client.get_root_span( pipeline_run_id=self.id)
        if span is not None:
            span_context = SpanContext(client=self.client, span=span)
        return span_context

    def get_details(self):
        """
            Description:
                Get root span of the pipeline run
        :return: SpanContext instance of the root span
        """
        response = self.client.get_run_details(pipeline_id=self.pipelineId, version_id=self.versionId)
        return response

    def get_spans(self):
        """
            Description:
                Get span of the pipeline run
        :return: SpanContext instance of the input span uid
        """
        spans = self.client.get_spans(pipeline_run_id=self.id)
        span_context_list = []
        for res in spans:
            span_context_list.append(SpanContext(client=self.client, span=res))
        return span_context_list


@dataclass
class Node:
    id = None
    type: NodeType = None
    assetId = None
    pipelineId = None
    name = None
    description = None
    context = None
    meta: PipelineMetadata = None
    versionId = None
    uid = None
    createdVia: NodeAddition = NodeAddition.PIPELINE_CREATION
    status: NodeStatus = NodeStatus.ACTIVE
    parentId = None

    def __init__(self,
                 id = None,
                 type: NodeType = None,
                 assetId = None,
                 pipelineId = None,
                 name = None,
                 description = None,
                 context = None,
                 meta: PipelineMetadata = None,
                 versionId = None,
                 uid = None,
                 createdVia: NodeAddition = NodeAddition.PIPELINE_CREATION,
                 status: NodeStatus = NodeStatus.ACTIVE,
                 parentId = None, *args, **kwargs
                 ):
        self.id = id
        self.type = type
        self.assetId = assetId
        self.pipelineId = pipelineId
        self.name = name
        self.description = description
        self.context = context
        if isinstance(meta, dict):
            self.meta = PipelineMetadata(**meta)
        else:
            self.meta = meta
        self.versionId = versionId
        self.uid = uid
        self.createdVia = createdVia
        self.status = status
        self.parentId = parentId

    def __repr__(self):
        return f"Node({self.__dict__})"


@dataclass 
class Edge:
    id = None
    fromId = None
    toId = None
    fromAssetType: NodeType = None
    toAssetType: NodeType = None
    versionId = None

    def __init__(self,
                 id = None,
                 fromId = None,
                 toId = None,
                 fromAssetType: NodeType = None,
                 toAssetType: NodeType = None,
                 versionId = None, *args, **kwargs
                 ):
        self.id = id
        self.fromId = fromId
        self.toId = toId
        self.fromAssetType = fromAssetType
        self.toAssetType = toAssetType
        self.versionId = versionId

    def __repr__(self):
        return f"Edge({self.__dict__})"


class Pipeline:
    def __init__(self, uid: str,
                 name: str,
                 description: str = None,
                 meta: PipelineMetadata = None,
                 createdAt: str = None,
                 enabled: bool = None,
                 id: int = None,
                 interrupt: bool = None,
                 notificationChannels=None,
                 schedule: str = None,
                 scheduled: bool = None,
                 schedulerType: str = None,
                 updatedAt=None,
                 context=None,
                 client=None,
                 tags=None,
                 *args, **kwargs):
        """
        Description:
            Class of the pipeline
        :param uid: uid of the pipeline
        :param name: name of the pipeline
        :param description: pipeline desc
        :param meta: (PipelineMetadata)metadata of the pipeline
        :param createdAt: creation time of pipeline
        :param enabled: True if pipeline is interrupted else false
        :param id: pipeline id
        :param updatedAt: updated time of the given pipeline
        :param context: context data for the pipeline (dir)
        :param kwrgs:
        """
        self.uid = uid
        self.name = name
        self.createdAt = createdAt
        self.enabled = enabled
        self.interrupt = interrupt
        self.notificationChannels = notificationChannels
        self.schedule = schedule
        self.scheduled = scheduled
        self.schedulerType = schedulerType
        self.id = id
        self.updatedAt = updatedAt
        self.description = description
        self.context = context
        if isinstance(meta, dict):
            self.meta = PipelineMetadata(**meta)
        else:
            self.meta = meta

        self.tags = list()
        if tags is not None:
            for obj in tags:
                if isinstance(obj, dict):
                    self.tags.append(Tag(**obj))
                else:
                    self.tags.append(obj)
        self.client = client

    def __repr__(self):
        return f"Pipeline({self.__dict__})"

    def __eq__(self, other):
        return self.uid == other.uid

    # convert job object to dict type
    def _convert_job_to_dict(self, job: CreateJob):
        """
        Description:
            Convert createJob class instance to dict type
        :param job: createJob class instance
        :return: dict form of createJob class instance
        """
        job_dict = job.__dict__
        if hasattr(job, 'meta'):
            meta = asdict(job.meta)
            job_dict['meta'] = meta
        input_dict = []
        for ds_in in job.inputs:
            input_dict.append(asdict(ds_in, dict_factory=lambda x: {k: v for (k, v) in x if v is not None}))
        output_dict = []
        for ds_out in job.outputs:
            output_dict.append(asdict(ds_out, dict_factory=lambda x: {k: v for (k, v) in x if v is not None}))
        job_dict['inputs'] = input_dict
        job_dict['outputs'] = output_dict
        pipeline_run = self.client.get_pipeline_run(job.pipeline_run_id)
        job_dict['version'] = pipeline_run.versionId
        return job_dict

    # to create job for any given pipeline
    def create_job(self, job: CreateJob):
        """
        Description:
            Used to create job and span in a pipeline
        :param job: createJob class instance that you want to add in pipeline
        :return: Job class instance of created job
        """
        if job.uid is None or job.name is None or job.pipeline_run_id is None:
            raise Exception('To create a job job uid, name and pipeline_run_id is required')

        payload = self._convert_job_to_dict(job)
        res = self.client.create_job(payload, self.id)
        if job.bounded_by_span:
            if job.pipeline_run_id is not None and job.span_uid is not None:
                associated_job_uids = [job.uid]
                span_uid_temp = job.span_uid
                # Get root span to create child span under
                pipeline_run = self.get_run(job.pipeline_run_id)
                parent_span_context = pipeline_run.get_root_span()
                if parent_span_context is not None:
                    # Create child span under root span
                    span_context = parent_span_context.create_child_span(
                        uid=span_uid_temp,
                        context_data={
                            'time': str(datetime.now())
                        },
                        associatedJobUids=associated_job_uids
                    )
                else:
                    # Create root span under pipeline_run as root span doesn't exist
                    pipeline_run.create_span(
                        uid=span_uid_temp,
                        context_data={
                            'time': str(datetime.now())
                        },
                        associatedJobUids=associated_job_uids
                    )
            else:
                raise Exception('To create a span span_uid and pipeline_run_id are required')
        return res

    # to delete pipeline
    def delete(self):
        """
        Description:
            Used to delete a pipeline
        """
        res = self.client.delete_pipeline(self.id)
        return res

    # convert pipeline run to dict type
    def _convert_pipeline_run_to_dict(self, pipeline_run: PipelineRun):
        """
            Description:
                Convert PipelineRun class instance to dict type
            :param pipeline_run: PipelineRun class instance
            :return: dict form of PipelineRun class instance
        """
        payload = pipeline_run.__dict__
        payload['status'] = pipeline_run.status.name
        if pipeline_run.result is not None:
            payload['result'] = pipeline_run.result.name
        else:
            del payload['result']
        pipeline_run_payload = {'run': payload}
        return pipeline_run_payload

    # create run for a pipeline
    def create_pipeline_run(self, context_data: {} = None, continuation_id: str = None) -> PipelineRun:
        """
        Description:
            used to create a pipeline run
        :param context_data: pipeline run argument
        :param continuation_id: continuation_id of pipeline run. Default value is None
        :return: pipelineRun class instance
        """
        create_pipeline_run = PipelineRun(
            pipelineId=self.id,
            args=context_data,
            continuationId=continuation_id
        )
        payload = self._convert_pipeline_run_to_dict(create_pipeline_run)
        res = self.client.create_pipeline_run(payload)
        return res

    def get_latest_pipeline_run(self) -> PipelineRun:
        return self.client.get_latest_pipeline_run(pipeline_id=self.id)

    def get_run(self, pipeline_run_id=None, continuation_id=None) -> PipelineRun:
        return self.client.get_pipeline_run(
            pipeline_run_id=pipeline_run_id,
            continuation_id=continuation_id,
            pipeline_id=self.id
        )

    def get_runs(self) -> List[PipelineRun]:
        return self.client.get_pipeline_runs(pipeline_id=self.id)


class PipelineInfo:
    pipeline: Pipeline = None
    runs: List[PipelineRun] = list()

    def __init__(self, pipeline=None, runs=None, *args, **kwargs):
        if isinstance(pipeline, dict):
            self.pipeline = Pipeline(**pipeline)
        else:
            self.pipeline = pipeline
        self.runs = list()
        if runs is not None:
            for obj in runs:
                if isinstance(obj, dict):
                    obj['client'] = self.pipeline.client
                    self.runs.append(PipelineRun(**obj))
                else:
                    self.runs.append(obj)

    def __repr__(self):
        return f"PipelineInfo({self.__dict__})"


class PipelineDetails:
    pipeline: Pipeline = None
    nodes: List[Node] = list()
    edges: List[Edge] = list()

    def __init__(self, pipeline=None, nodes=None, edges=None, *args, **kwargs):
        if isinstance(pipeline, dict):
            self.pipeline = Pipeline(**pipeline)
        else:
            self.pipeline = pipeline
        self.nodes = list()
        if nodes is not None:
            for obj in nodes:
                if isinstance(obj, dict):
                    self.nodes.append(Node(**obj))
                else:
                    self.nodes.append(obj)
        self.edges = list()
        if edges is not None:
            for obj in edges:
                if isinstance(obj, dict):
                    self.edges.append(Edge(**obj))
                else:
                    self.edges.append(obj)

    def __repr__(self):
        return f"PipelineDetails({self.__dict__})"



from enum import Enum, IntEnum, auto
from dataclasses import dataclass


class RuleExecutionStatus(Enum):
    STARTED = 1
    RUNNING = 2
    ERRORED = 3
    WARNING = 4
    SUCCESSFUL = 5
    ABORTED = 6


class FailureStrategy(IntEnum):
    DoNotFail = auto()
    FailOnError = auto()
    FailOnWarning = auto()


class PolicyType(Enum):
    DATA_QUALITY = 'DATA-QUALITY'
    RECONCILIATION = 'RECONCILIATION'


class AssetSourceType(Enum):
    BIGQUERY = 'BIGQUERY'
    REDSHIFT = 'REDSHIFT'
    SNOWFLAKE = 'SNOWFLAKE'
    TERADATA = 'TERADATA'
    HIVE = 'HIVE'
    HBASE = 'HBASE'
    AZURE_MSSQL = 'AZURE_MSSQL'
    AZURE_DATALAKE = 'AZURE_DATALAKE'
    AWS_GLUE = 'AWS_GLUE'
    AWS_S3 = 'AWS_S3'
    HDFS = 'HDFS'
    GCS = 'GCS'
    KAFKA = 'KAFKA'
    MYSQL = 'MYSQL'
    MEMSQL = 'MEMSQL'
    POSTGRESQL = 'POSTGRESQL'
    TABLEAU = 'TABLEAU'
    ORACL = 'ORACLE'
    AWS_ATHENA = 'AWS_ATHENA'
    DATABRICKS = 'DATABRICKS'
    MONGO = 'MONGO'
    MODEL_BAG = 'MODEL_BAG'
    FEATURE_BAG = 'FEATURE_BAG'
    PRESTO = 'PRESTO'
    DB2 = 'DB2'
    CLICKHOUSE = 'CLICKHOUSE'
    VIRTUAL_DATASOURCE = 'VIRTUAL_DATASOURCE'


DATA_QUALITY = 'DATA_QUALITY'
RECONCILIATION = 'RECONCILIATION'

# MIN_TORCH_BACKEND_VERSION_SUPPORTED is the minimum torch backend version supported by SDK.

MIN_TORCH_BACKEND_VERSION_SUPPORTED = '2.2.0'

@dataclass
class PipelineBulkupdateRequest:
    dags: list()
    dagruns: list()
    tasks: list()

@dataclass
class TorchBuildVersion:
    buildVersion = None
    buildDate = None

    def __init__(self, buildVersion=None, buildDate=None, *args, **kwargs):
        self.buildVersion = buildVersion
        self.buildDate = buildDate

    def __repr__(self):
        return f"TorchBuildVersion({self.__dict__})"


@dataclass
class SdkSupportedVersions:
    maxVersion = None
    minVersion = None

    def __init__(self, maxVersion=None, minVersion=None, *args, **kwargs):
        self.maxVersion = maxVersion
        self.minVersion = minVersion

    def __repr__(self):
        return f"SdkSupportedVersions({self.__dict__})"